# -*- coding: utf-8 -*-
"""Python lager brewed by a loguru"""
from lager import logging
from lager._version import (
    VERSION_INFO,
    VERSION_MAJOR,
    VERSION_MINOR,
    VERSION_PATCH,
    __version__,
)
from lager.const import LAGER_PORT, LOGURU_DEFAULT_FMT, TORNADO_FMT
from lager.core import LN, LOG, flog, handlers, ln, log, logger, loglevel


__all__ = [
    'LAGER_PORT',
    'VERSION_MAJOR',
    'VERSION_MINOR',
    'VERSION_PATCH',
    'VERSION_INFO',
    '__version__',
    'LOGURU_DEFAULT_FMT',
    'TORNADO_FMT',
    'loglevel',
    'logger',
    'LOG',
    'log',
    'LN',
    'ln',
]



# -*- coding: utf-8 -*-
"""FastAPI logging"""
from lager.logging import intercept


FASTAPI_LOGGERS = [
    'gunicorn',
    'gunicorn.errors' 'uvicorn',
    'uvicorn.error',
    'fastapi',
    'sqlalchemy',
]


def fastapi_intercept() -> None:
    intercept(FASTAPI_LOGGERS)



# -*- coding: utf-8 -*-
"""Constants go here!"""
LAGER_PORT = 52437

TORNADO_FMT = "".join(
    [
        "<level>",
        "[{level.name[0]} ",
        "{time:YYMMDDTHH:mm:ss} ",
        "{name}:{module}:{line}]",
        "</level> ",
        "{message}",
    ]
)

LOGURU_DEFAULT_FMT = "".join(
    [
        "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green>",
        " | ",
        "<level>{level: <8}</level>",
        " | ",
        "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan>",
        " - ",
        "<level>{message}</level>",
    ]
)

LOG_LEVELS = {
    "notset": "NOTSET",
    "n": "NOTSET",
    "debug": "DEBUG",
    "d": "DEBUG",
    "info": "INFO",
    "i": "INFO",
    "s": "SUCCESS",
    "success": "SUCCESS",
    "warning": "WARNING",
    "warn": "WARNING",
    "w": "WARNING",
    "error": "ERROR",
    "e": "ERROR",
    "critical": "CRITICAL",
    "fatal": "CRITICAL",
    "c": "CRITICAL",
    # enum/enum-strings
    "0": "NOTSET",
    "10": "DEBUG",
    "20": "INFO",
    "25": "SUCCESS",
    "30": "WARNING",
    "40": "ERROR",
    "50": "CRITICAL",
}



# -*- coding: utf-8 -*-

import httpx


SINKS = []


class HttpxSink(object):
    def __init__(self, url, *args, **kwargs):
        self.url = url
        self.client = httpx.AsyncClient(*args, **kwargs)
        SINKS.append(self)

    async def __call__(self, msg):
        httpx.post(self.url, data={'msg': msg})

    async def handle(self, message):
        await self.client.post(url=self.url, data={'msg': message})

    async def await_delete_channels(self):
        await self.client.aclose()



# -*- coding: utf-8 -*-
"""Lager/Loguru + std logging"""
import logging

from typing import Any, Dict, List

from lager.core import LOG, loglevel


class StdLoggingHandler(logging.Handler):
    """Logging intercept handler"""

    def emit(self, record: logging.LogRecord) -> None:
        # Get corresponding Loguru level if it exists
        try:
            level = LOG.level(record.levelname).name
        except AttributeError:
            level = loglevel(record.levelno)

        # Find caller from where originated the logging call
        frame = logging.currentframe()
        depth = 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back  # type: ignore
            depth += 1

        LOG.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())


def _logger_dict() -> Any:
    return logging.root.manager.loggerDict  # type: ignore


def loggers_dict() -> Dict[str, logging.Logger]:
    return {name: logging.getLogger(name) for name in _logger_dict()}  # type: ignore


def intercept(loggers: List[str]) -> None:
    for logger in loggers:
        std_logger = logging.getLogger(logger)
        std_logger.handlers = [StdLoggingHandler()]


def intercept_all() -> None:
    intercept(list(loggers_dict().keys()))



# -*- coding: utf-8 -*-
"""Python lager brewed by a loguru"""
import asyncio
import atexit as _atexit
import sys as _sys

from functools import wraps
from time import time
from typing import Any, Dict, Union

from loguru import _defaults, logger
from loguru._handler import Handler
from loguru._logger import Core as _Core, Logger as _Logger

from lager.const import LOG_LEVELS


__all__ = ['loglevel', 'flog', 'handlers', 'logger', 'log', 'LOG', 'ln', 'LN']

try:
    import orjson  # type: ignore

    def _stringify_new_line(serializable: Any) -> str:
        return orjson.dumps(serializable, option=orjson.OPT_APPEND_NEWLINE).decode(
            'utf-8'
        )

    def _stringify_no_new_line(serializable: Any) -> str:
        return f"{orjson.dumps(serializable).decode('utf-8')}\n"

    _stringify = (
        _stringify_new_line
        if hasattr(orjson, "OPT_APPEND_NEWLINE")
        else _stringify_no_new_line
    )

    def _serialize_record(text: str, record: Dict[str, Any]) -> str:
        exception = record["exception"]

        if exception is not None:
            exception = {
                "type": None if exception.type is None else exception.type.__name__,
                "value": exception.value,
                "traceback": bool(record["exception"].traceback),
            }

        serializable = {
            "text": text,
            "record": {
                "elapsed": {
                    "repr": record["elapsed"],
                    "seconds": record["elapsed"].total_seconds(),
                },
                "exception": exception,
                "extra": record["extra"],
                "file": {"name": record["file"].name, "path": record["file"].path},
                "function": record["function"],
                "level": {
                    "icon": record["level"].icon,
                    "name": record["level"].name,
                    "no": record["level"].no,
                },
                "line": record["line"],
                "message": record["message"],
                "module": record["module"],
                "name": record["name"],
                "process": {"id": record["process"].id, "name": record["process"].name},
                "thread": {"id": record["thread"].id, "name": record["thread"].name},
                "time": {
                    "repr": record["time"],
                    "timestamp": record["time"].timestamp(),
                },
            },
        }
        return _stringify(serializable)

    Handler._serialize_record = staticmethod(_serialize_record)
except ModuleNotFoundError:
    pass

logger = _Logger(_Core(), None, 0, False, False, False, False, True, None, {})
if _defaults.LOGURU_AUTOINIT and _sys.stderr:
    logger.add(_sys.stderr)

_atexit.register(logger.remove)

# commonly used dgpy aliases
log = logger
LOG = logger
# ln => natural log
ln = logger
LN = logger


def loglevel(level: Union[str, int]) -> str:
    """Convert log-level abrev to a valid loguru log level"""
    return LOG_LEVELS[str(level).strip("'").strip('"').lower()]


def flog(funk=None, level: str = "debug", enter: bool = True, exit: bool = True):
    """Log function (sync/async) enter and exit using this decorator

    Args:
        funk (Callable): Function to decorate
        level (Union[int, str]): Log level
        enter (bool): Log function entry if True
        exit (bool): Log function exit if False

    Returns:
        A wrapped function that now has logging!

    Usage:
        # SYNC
        @flog
        def add(a, b):
            return a + b
        add(1, 4)

        # ASYNC
        @flog
        async def add_async(a, b):
            return a + b
        import asyncio
        asyncio.run(add_async(1, 4))

    """

    def _flog(funk):
        name = funk.__name__

        @wraps(funk)
        def _flog_decorator(*args, **kwargs):
            logger_ = logger.opt(depth=1)
            if enter:
                logger_.log(
                    loglevel(level),
                    "FLOG-ENTER > '{}' (args={}, kwargs={})",
                    name,
                    args,
                    kwargs,
                )
            ti = time()
            result = funk(*args, **kwargs)
            tf = time()
            if exit:
                logger_.log(
                    loglevel(level),
                    "FLOG-EXIT < '{}' (return={}, dt_sec={})",
                    name,
                    result,
                    tf - ti,
                )
            return result

        @wraps(funk)
        async def _flog_decorator_async(*args, **kwargs):
            logger_ = logger.opt(depth=7)
            if enter:
                logger_.log(
                    loglevel(level),
                    "FLOG-ENTER > '{}' (args={}, kwargs={})",
                    name,
                    args,
                    kwargs,
                )
            ti = time()
            result = await funk(*args, **kwargs)
            tf = time()
            if exit:
                logger_.log(
                    loglevel(level),
                    "FLOG-EXIT < '{}' (return={}, dt_sec={})",
                    name,
                    result,
                    tf - ti,
                )
            return result

        if asyncio.iscoroutinefunction(funk) or asyncio.iscoroutine(funk):
            return _flog_decorator_async
        return _flog_decorator

    return _flog(funk) if funk else _flog


def handlers() -> Dict[int, Handler]:
    """Return all handlers"""
    return logger._core.handlers  # type: ignore



# -*- coding: utf-8 -*-
"""Lager & pydantic"""
from datetime import datetime, timedelta
from types import TracebackType
from typing import Optional, Type, Union

from jsonbourne.pydantic import JsonBaseModel


class RecordFile(JsonBaseModel):
    name: str
    path: str


class RecordLevel(JsonBaseModel):
    name: str
    no: int
    icon: str


class RecordThread(JsonBaseModel):
    id: int
    name: str


class RecordProcess(JsonBaseModel):
    id: int
    name: str


class RecordException(JsonBaseModel):
    type: Optional[Type[BaseException]]
    value: Optional[BaseException]
    traceback: Optional[TracebackType]


class Record(JsonBaseModel):
    elapsed: timedelta
    exception: Optional[RecordException]
    extra: dict
    file: RecordFile
    function: str
    level: RecordLevel
    line: int
    message: str
    module: str
    name: Union[str, None]
    process: RecordProcess
    thread: RecordThread
    time: datetime


class Message(JsonBaseModel):
    record: Record



# -*- coding: utf-8 -*-
"""`lager` version"""
__version__ = "0.12.0"
VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH = [int(el) for el in __version__.split('.')]
VERSION_INFO = (VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH)

__all__ = [
    "VERSION_MAJOR",
    "VERSION_MINOR",
    "VERSION_PATCH",
    "VERSION_INFO",
    "__version__",
]
